#!/s/bin/python -- # -*- python -*-
# $Id: kstat.py,v 1.4 1999-12-16 16:26:41-06 annis Exp $
# $Source: /u/annis/code/NewMom/kstat/RCS/kstat.py,v $
#
# Copyright (c) 1998 William S. Annis.  All rights reserved.  This
# is free software; you can redistribute it and/or modify it under the
# same terms as Perl (the Artistic Licence).

"""OOPy interface to the Solaris kstat API.

This interface is simply an encapsulation of chain traversal and data
extraction.  You simply register kstat records you're interested in,
and every time you update the instance, all registered records will be
extracted.

See the documentation for the watch method for an explanation of the
registration process and extraction process.

In typical use, a single Kstat() will be instantiated, several
interesting kstat records registered to be watched, then the chain
will be updated periodically and data extracted.
"""

import UserList, UserDict, time, os
from wkstat import *


# This is a mapping of kstat names to their conversion functions for
# RAW kstats.  This is used by the match method of Kstat.
_rawcnv = {'vminfo': wkstat_vminfo_read,
           'sysinfo': wkstat_sysinfo_read,
           'cpu_stat': wkstat_cpustat_read,
           'ncstats': wkstat_ncstats_read}

class KstatError(Exception):
    def __repr__(self):
        return self.__str__()


class KstatNamed(UserDict.UserDict):
    def __init__(self, record, dict=None):
        self.record = record
        UserDict.UserDict.__init__(self, dict)
    def id(self):
        """Return the full name (module, instance and name) of this kstat."""
        return record

class KstatIO(UserDict.UserDict):
    def __init__(self, record, dict=None):
        self.record = record
        UserDict.UserDict.__init__(self, dict)
    def id(self):
        """Return the full name (module, instance and name) of this kstat."""
        return record


class Kstat:
    """OO interface to the Solaris kstat(3K) library."""

    def __init__(self):
        self.k = wkstat_open()
        self.state_change = 0
        self.watch_for = []
        self.values = None

    def update_chain(self):
        if wkstat_update(self.k):
            self.state_change = 1
        else:
            self.state_change = 0

    def update(self):
        """Scan the kstat chain for updated information.

        kstat records not of type KSTAT_TYPE_NAMED, KSTAT_TYPE_IO or
        KSTAT_TYPE_RAW are ignored.  So, no INTR types for now.  
        Only a few RAW kstats are currently supported: vminfo, sysinfo,
        cpu_stat and ncstats.
        """
        self.update_chain()
        wkstat_rewind(self.k)
        self.values = {}
        more = 1
        while more >= 1:
            for (id, mod, inst, name) in self.watch_for:
                match = self._match(mod, inst, name)
                if match:
                    wkstat_read(self.k)
                    type = wkstat_type(self.k)
                    if type == WKSTAT_TYPE_NAMED:
                        self._add_value(id,
                                        self._load_named(match))
                    elif type == WKSTAT_TYPE_IO:
                        self._add_value(id, KstatIO(match,
                                                    wkstat_io_read(self.k)))
                    elif type == WKSTAT_TYPE_RAW:
                        if _rawcnv.has_key(name):
                            self._add_value(id, apply(_rawcnv[name],(self.k,)))
                        elif _rawcnv.has_key(mod):
                            # This is for cpu_stat, since the name values
                            # for this will be 'cpu_stat0', etc...
                            self._add_value(id,
                                            apply(_rawcnv[mod], (self.k,)))
                        else:
                            self._add_value(id, "No RAW mapping for " + `name`)
                    else:
                        #self._add_value(id, "Unsupported kstat type")
                        # IGNORE THESE FOR NOW
                        pass

            more = wkstat_next(self.k)

    def _match(self, mod, inst, name):
        """Match the current kstat record against module, instance and name."""
        s_inst= wkstat_instance(self.k)
        if inst:
            if s_inst != inst:
                return 0

        s_name = wkstat_name(self.k)
        if name:
            if s_name != name:
                return 0

        s_mod = wkstat_module(self.k)
        if mod:
            if s_mod != mod:
                return 0

        return (s_mod, s_inst, s_name)

    def _add_value(self, id, val):
        if not self.values.has_key(id):
            self.values[id] = []

        self.values[id].append(val)            

    def _load_named(self, record):
        """Load all of the names of a named record into a dictionary."""
        rec = {}
        n = wkstat_named_read(self.k)
        while n:
            rec[n[0]] = n[2]
            n = wkstat_named_read(self.k)

        return KstatNamed(record, rec)

    def watch(self, module, instance, name, id=None):
        """Register kstat information to watch for.

        k.watch(module, instance, name {, id})

        Each record registered has a unique id which is returned by the
        call to this method:

            load_id = k.watch("unix", None, "system_misc")

        After the chain has been updated, the id is used with the get
        method to extract the information.  You may also assign a more
        meaningful ID:

            k.watch("unix", None, "system_misc", id="loads")

        For each of the three arguments, a value of 'None' means to
        check for all values of that field.  So, in the load_id line
        above, all instances with a module of 'unix' and name of
        'system_misc' will be stored on each sweep through the kstat
        chain.  Thus, the get method may return a list of answers for
        a given id.
        """
        if not id:
            id = `module`+`instance`+`name`
        self.watch_for.append((id, module, instance, name,))
        return id

    def get(self, id):
        """Get the values from the last sweep for a given id.

        For the example from the watch documentation:

            data = k.get(load_id)

        This will put into data a list of records.  These records will be
        either KstatNamed or KstatIO instances.  Each of these classes has
        a method, id, which will return a tuple of (module, instance, name)
        for identification when a single watch specification results in
        multiple matchs.  So, data[0].id() should return something like
        ('unix', 0, 'system_misc').
        """
        return self.values[id]


# EOF
